/***********************************************************************************
* T-SQL: Simple Changes That Go a Long Way
* Dave Valentine
* COALSCE and ISNULL Functions Demo SQL Script
***********************************************************************************/

USE AdventureWorks2012;
GO

--SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO


/***********************************************************************************
* PART I
*
* Show the difference between ISNULL and COALESCE
*
* First create a temp table to store some data
***********************************************************************************/
IF OBJECT_ID('tempdb..#TEMP_NULL') IS NOT NULL
    DROP TABLE #TEMP_NULL;

CREATE TABLE #TEMP_NULL
    (
     IntColumn INT NULL
   , DecimalColumn DECIMAL(4, 2)
    );


/***********************************************************************************
* Insert a NULL and decimal value into the temp table
***********************************************************************************/
INSERT  INTO #TEMP_NULL
        (IntColumn, DecimalColumn)
VALUES  (NULL, 40.2);

/***********************************************************************************
* Select the inserted record notice difference between the 
* ISNULL and COALESCE functions
***********************************************************************************/
SELECT  UsingISNULL = ISNULL(IntColumn, DecimalColumn)
      , UsingCOALESCE = COALESCE(IntColumn, DecimalColumn)
FROM    #TEMP_NULL tn;





/***********************************************************************************
* Insert another record this time with an INT and a decimal 
* value into the temp table
***********************************************************************************/
INSERT  INTO #TEMP_NULL
        (IntColumn, DecimalColumn)
VALUES  (30, 30.5);

/***********************************************************************************
* Select the inserted records notice difference between the 
* ISNULL and COALESCE functions
***********************************************************************************/
SELECT  UsingISNULL = ISNULL(IntColumn, DecimalColumn)
      , UsingCOALESCE = COALESCE(IntColumn, DecimalColumn)
FROM    #TEMP_NULL tn;








/***********************************************************************************
* PART II
*
* Build a comma seperated list of values using COALESE, and the magic behind the 
* function...
*
* First lets find a list of all people with the last name Kennedy.
***********************************************************************************/
SELECT  p.FirstName + ' ' + p.LastName
FROM    Person.Person p
WHERE   p.LastName = 'Kennedy';





/***********************************************************************************
* Lets create a variable to store the comma seperated list of values,
* then store the first and last names in that variable.
*
* Notice that we only see the last record in the list?
***********************************************************************************/
DECLARE @FLNameCSV NVARCHAR(500);

SELECT  @FLNameCSV = p.FirstName + ' ' + p.LastName
FROM    Person.Person p
WHERE   p.LastName = 'Kennedy';

SELECT  @FLNameCSV;




/***********************************************************************************
* Lets add the variable with a comma to the first and last names 
* on each iteration
*
* The result is NULL...
***********************************************************************************/
DECLARE @FLNameCSV NVARCHAR(500);

SELECT  @FLNameCSV = @FLNameCSV + ', ' + p.FirstName + ' ' + p.LastName
FROM    Person.Person p
WHERE   p.LastName = 'Kennedy';

SELECT  @FLNameCSV;




/***********************************************************************************
* Lets our variable a default value
*
* Now we have a comma seperated list, with an extra comma.
***********************************************************************************/
DECLARE @FLNameCSV NVARCHAR(500) = '';

SELECT  @FLNameCSV = @FLNameCSV + ', ' + p.FirstName + ' ' + p.LastName
FROM    Person.Person p
WHERE   p.LastName = 'Kennedy';

SELECT  @FLNameCSV;



/***********************************************************************************
* Lets add a substring method to remove the extra comma and space.
*
* Now we have a comma seperated list of values with out 
* extra commas.
***********************************************************************************/
DECLARE @FLNameCSV NVARCHAR(500) = '';

SELECT  @FLNameCSV = @FLNameCSV + ', ' + p.FirstName + ' ' + p.LastName
FROM    Person.Person p
WHERE   p.LastName = 'Kennedy';

SELECT  SUBSTRING(@FLNameCSV, 3, LEN(@FLNameCSV));




/***********************************************************************************
* Lets add the magic of the COALSCE function...
*
* Poof a comma seperated list of values
***********************************************************************************/
DECLARE @FLNameCSV NVARCHAR(500);

SELECT  @FLNameCSV = COALESCE(@FLNameCSV + ', ', '') + p.FirstName + ' ' + p.LastName
FROM    Person.Person p
WHERE   p.LastName = 'Kennedy';

SELECT  @FLNameCSV;


